"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fm = require("front-matter");
const marked = require("marked");
const globby = require("globby");
const util = require("./util");
const get_image_meta_1 = require("./get-image-meta");
const getDirName = (pathName) => path
    .dirname(pathName)
    .split(path.sep)
    .pop();
const getDataForImage = (albumsDir, albumName) => (file) => ({
    filePath: file,
    detailLinkProps: util.getDetailLinkProps(albumsDir, albumName, file),
    fileName: path.basename(file),
});
const getMetaFromImage = (image) => __awaiter(this, void 0, void 0, function* () {
    const meta = yield get_image_meta_1.default(image.filePath);
    return Object.assign({}, image, { meta });
});
const getImages = (albumsDir, albumName) => __awaiter(this, void 0, void 0, function* () {
    const pattern = path.join('static', albumsDir, albumName, '*.jpg');
    const files = yield globby(pattern);
    const images = files.map(getDataForImage(albumsDir, albumName));
    return yield Promise.all(images.map(getMetaFromImage));
});
const getAlbumContent = (albumsDir, albumPath) => __awaiter(this, void 0, void 0, function* () {
    const content = yield util.pReadFile(albumPath);
    const { attributes, body } = fm(String(content));
    const name = getDirName(albumPath);
    return {
        meta: attributes,
        path: `/${albumsDir}/${name}/`,
        body: marked(body),
        name,
    };
});
const gatherAlbumData = (albumsDir) => (file) => __awaiter(this, void 0, void 0, function* () {
    const albumName = file
        .split(path.sep)
        .splice(-2)
        .shift();
    const [content, images] = yield Promise.all([
        getAlbumContent(albumsDir, file),
        getImages(albumsDir, albumName),
    ]);
    return {
        content,
        images,
    };
});
const getPageContent = (contentDir) => (filePath) => __awaiter(this, void 0, void 0, function* () {
    const content = yield util.pReadFile(filePath);
    const { attributes, body } = fm(String(content));
    const regExp = new RegExp(`^.*${contentDir}${path.sep}|${path.sep}?index.md$`, 'g');
    return {
        [filePath.replace(regExp, '') || 'index']: {
            meta: attributes,
            name: getDirName(filePath),
            body: marked(body),
        },
    };
});
const getPages = (pagesGlob, albumsGlob, contentDir) => __awaiter(this, void 0, void 0, function* () {
    const files = yield globby(pagesGlob, { ignore: albumsGlob });
    const pages = yield Promise.all(files.map(getPageContent(contentDir)));
    return pages.reduce((a, b) => Object.assign(a, b), {});
});
exports.default = (config) => __awaiter(this, void 0, void 0, function* () {
    const patterns = util.getGlobPatterns(config);
    const albumFiles = yield globby(patterns.albums);
    const albums = yield Promise.all(albumFiles.map(gatherAlbumData(config.albumsDir)));
    const pages = yield getPages(patterns.pages, patterns.albums, config.contentDir);
    return { albums, pages };
});
