"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const fs_1 = require("fs");
const path = require("path");
const mkdirp = require("mkdirp");
const got = require("got");
const config_1 = require("./config");
const pWriteFile = util_1.promisify(fs_1.writeFile);
const pMkdirp = util_1.promisify(mkdirp);
const getDataPaths = (paths, baseUrl, albumsDir, outDir) => Object.keys(paths)
    .filter((x) => x !== '/')
    .map((x) => x.replace(/(\/|\b)$/, '.json'))
    .map((x) => ({
    source: `${baseUrl}${x}`,
    destination: path.join(outDir, 'data', x),
}))
    .concat({
    source: `${baseUrl}/index.json`,
    destination: path.join(outDir, 'data', 'index.json'),
});
const fetchData = (dataItem) => __awaiter(this, void 0, void 0, function* () {
    const { body } = yield got(dataItem.source);
    return Object.assign({}, dataItem, { contents: body || '{}' });
});
const createAlbumFolders = (albumsDir) => (dataItem) => __awaiter(this, void 0, void 0, function* () {
    if (!dataItem.destination.includes(`/${albumsDir}/`)) {
        return;
    }
    const isAlbum = new RegExp(`/${albumsDir}/[^/]+\.json$`);
    if (isAlbum.test(dataItem.destination)) {
        return;
    }
    const albumFolderPath = dataItem.destination.replace(/[^/]+\.json$/, '');
    yield pMkdirp(albumFolderPath);
});
const writeData = (dataItem) => __awaiter(this, void 0, void 0, function* () {
    yield pWriteFile(dataItem.destination, dataItem.contents, 'utf8');
});
exports.default = (paths) => __awaiter(this, void 0, void 0, function* () {
    const conf = config_1.default();
    const baseUrl = `http://${conf.hostname}:${conf.port}/data`;
    yield pMkdirp(`${conf.outDir}/data/${conf.albumsDir}`);
    const dataPaths = getDataPaths(paths, baseUrl, conf.albumsDir, conf.outDir);
    const dataItems = yield Promise.all(dataPaths.map(fetchData));
    yield Promise.all(dataPaths.map(createAlbumFolders(conf.albumsDir)));
    yield Promise.all(dataItems.map(writeData));
});
